/*

 Copyright (C) 2002 Andrew Brown

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or any
 later version.

 This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import jm.JMC;
import jm.music.data.*;
import jm.music.tools.Mod;
import jm.util.*;
import qt.*;

/**
* This class describes one part for the NAIM program.
* The program can potentially have as many instances of this parrt
* as required.
* @author Andrew Brown
*/

public class NPart implements JMC {
	private Part part;
	private int[] pitches;// = {C4, E4, G4, A4, D4, F4, E4};
	private double[] rhythms;// = {Q, Q, C};
	private double[] durations; // = {0.8, 0.7, 0.4, 0.3};
	private int[] dynamics;// = {100, 70, 100, 120, 60};
	private double[] pans = {0.5};// = {0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0};
	private double partLength = 40.0;
	private double fadeInLength = 12.0;
	private double fadeOutLength = 12.0;
	private double nPartStartTime; // = 0.0;
	private int partInstrument = 0;
	
	public NPart(Part p) {
	    this.part = p;
	}
	
	public void composePart( NTabPanel ntp) {
	    // make a pharse for this part
	    part.removeAllPhrases();
	    gatherData(ntp);
	    part.addPhrase(makePhrase(ntp));
	}
	
	private void gatherData( NTabPanel ntp ) {
	    part.setInstrument(Integer.valueOf(ntp.instrumentValue.getText()).intValue());
	    nPartStartTime = Double.valueOf(ntp.startTimeValue.getText()).doubleValue();
	    partLength = Double.valueOf(ntp.lengthValue.getText()).doubleValue();
	    fadeInLength = Double.valueOf(ntp.fadeInValue.getText()).doubleValue();
	    fadeOutLength = Double.valueOf(ntp.fadeOutValue.getText()).doubleValue();
	    pitches = parseIntString((String)ntp.pitchValues.getText());
	    if (ntp.pitchOrderGroup.getSelection().getActionCommand() == "RandomLoop") pitches = shuffleInt(pitches);
	    rhythms = parseDoubleString((String)ntp.rhythmValues.getText());
	    if (ntp.rhythmOrderGroup.getSelection().getActionCommand() == "RandomLoop") rhythms = shuffleDouble(rhythms);
	    dynamics = parseIntString((String)ntp.dynamicValues.getText());
	    if (ntp.dynamicOrderGroup.getSelection().getActionCommand() == "RandomLoop") dynamics = shuffleInt(dynamics);
	    durations = parseDoubleString((String)ntp.durationValues.getText());
	    if (ntp.durationOrderGroup.getSelection().getActionCommand() == "RandomLoop") durations = shuffleDouble(durations);
	    //pans = parseDoubleString((String)ntp.panValues.getText());
	    //if (ntp.panOrderGroup.getSelection().getActionCommand() == "RandomLoop") pans = shuffleDouble(pans);
    }
    
    private int[] shuffleInt(int[] array) {
        // set up a new phrase
	    int[] tempArray = new int[array.length];

	    // put one element in to start off the new array
        tempArray[0] = array[(int)(Math.random()*array.length)];

	    // create the rest of the order
        for(int i=1;i<array.length;) {
	        //select a new note from this phrase
            int n = array[(int)(Math.random()* array.length)];
	        //check if  it is a note already used
	        boolean notUsed = true;
			for (int j=0; j< tempArray.length; j++) {
				if ( n == tempArray[j]) {
					notUsed = false;
				}
			}

			// if the note is not already present then add it to the phrase
			// and move to the next note in the phrase
			if (notUsed) {
			    tempArray[i] = n;
			    i++;
			}
		}
		return tempArray;
	 }
	 
	 private double[] shuffleDouble(double[] array) {
        // set up a new phrase
	    double[] tempArray = new double[array.length];

	    // put one element in to start off the new array
        tempArray[0] = array[(int)(Math.random() * array.length)];

	    // create the rest of the order
        for(int i=1;i<array.length;) {
	        //select a new note from this phrase
            double n = array[ (int)(Math.random() * array.length)];
	        //check if  it is a note already used
	        boolean notUsed = true;
			for (int j=0; j< tempArray.length; j++) {
				if ( n == tempArray[j]) {
					notUsed = false;
				}
			}

			// if the note is not already present then add it to the phrase
			// and move to the next note in the phrase
			if (notUsed) {
			    tempArray[i] = n;
			    i++;
			}
		}
		return tempArray;
	 }
    
    
    
    private int[] parseIntString(String s) {
        int[] tempArray = new int[100];
        String tempString;
        int firstPos, lastPos, count = 0;
        char c;
        for(int i = 0; i<s.length(); i++) {
            c = s.charAt(i);
            if(Character.isDigit(c)) {
                firstPos = i; // found first valid value
                i++;
                while( i<s.length() && Character.isDigit( s.charAt(i))) {
                    i++; 
                }
                lastPos = i; // found last pos
                tempString = s.substring(firstPos, lastPos);
                tempArray[count] = Integer.valueOf(tempString).intValue();
                count ++;
            }
         }
         int[] returnArray = new int[count];
         for (int j = 0;j < count; j++) {
            returnArray[j] = tempArray[j];
         }
         return returnArray;
    }
    
    private double[] parseDoubleString(String s) {
        double[] tempArray = new double[100];
        String tempString;
        int firstPos, lastPos, count = 0;
        char c;
        for(int i = 0; i<s.length(); i++) {
            c = s.charAt(i);
            if(Character.isDigit(c) || c == '.') {
                firstPos = i; // found first valid value
                i++;
                while( i<s.length() && 
                        (Character.isDigit( s.charAt(i)) || 
                        s.charAt(i) == '.')) {
                    i++; 
                }
                lastPos = i; // found last pos
                tempString = s.substring(firstPos, lastPos);
                tempArray[count] = Double.valueOf(tempString).doubleValue();
                count ++;
            }
         }
         double[] returnArray = new double[count];
         for (int j = 0;j < count; j++) {
            returnArray[j] = tempArray[j];
         }
         return returnArray;
    }
	
	private Phrase makePhrase( NTabPanel ntp) {
	    Phrase phr = new Phrase( nPartStartTime);
	    double rhy, dur, pan, beatCounter = 0.0;
	    int pit, dyn;
	    Note n = new Note();
        for(int i=0; beatCounter <= partLength; i++) {
            if (pitches.length > 0 && rhythms.length > 0) {
                if (ntp.pitchOrderGroup.getSelection().getActionCommand() == "AllRandom") {
                    pit  = pitches[(int)(Math.random() * pitches.length)];
                } else pit = pitches[i%pitches.length];
                if (ntp.rhythmOrderGroup.getSelection().getActionCommand() == "AllRandom") {
                    rhy = rhythms[(int)(Math.random() * rhythms.length)];
                } else rhy = rhythms[i%rhythms.length];
                n = new Note(pit, rhy);
                phr.addNote(n);
                beatCounter += rhythms[i%rhythms.length];
            } else break;
            if (durations.length > 0) {
                if (ntp.durationOrderGroup.getSelection().getActionCommand() == "AllRandom") {
                    dur = durations[(int)(Math.random() * durations.length)];
                } else dur = durations[i%durations.length];
                n.setDuration( rhythms[i%rhythms.length] * dur);
            }
            if (dynamics.length > 0) {
                if (ntp.dynamicOrderGroup.getSelection().getActionCommand() == "AllRandom") {
                    dyn = dynamics[(int)(Math.random() * dynamics.length)];
                } else dyn = dynamics[i%dynamics.length];
                n.setDynamic( dyn);
            }
            /*if (pans.length > 0) {
                if (ntp.panOrderGroup.getSelection().getActionCommand() == "AllRandom") {
                    pan = pans[(int)(Math.random() * pans.length)];
                } else pan = pans[i%pans.length];
                n.setPan( pan);
            }
            */
        }
	    Mod.fadeIn(phr, fadeInLength);
	    Mod.fadeOut(phr, fadeOutLength);
	    return phr;
	}
	    
}
